@@ -49,6 +49,7 @@ type ComponentUpgradePlan struct {
 	Name           string
 	CurrentVersion string
 	NewVersion     string
+	NodeName       string
 }
 
 // ComponentConfigVersionState describes the current and desired version of a component config
@@ -73,12 +74,61 @@ type ComponentConfigVersionState struct {
 
 // +k8s:deepcopy-gen:interfaces=k8s.io/apimachinery/pkg/runtime.Object
 
+// AvailableUpgrade represents information for a single available upgrade.
+type AvailableUpgrade struct {
+	metav1.TypeMeta
+
+	Description string
+
+	Components []ComponentUpgradePlan
+}
+
+// +k8s:deepcopy-gen:interfaces=k8s.io/apimachinery/pkg/runtime.Object
+
 // UpgradePlan represents information about upgrade plan for the output
 // produced by 'kubeadm upgrade plan'
 type UpgradePlan struct {
 	metav1.TypeMeta
 
-	Components []ComponentUpgradePlan
+	AvailableUpgrades []AvailableUpgrade
 
 	ConfigVersions []ComponentConfigVersionState
+
+	// TODO: Remove this field when v1alpha2 is removed
+	Components []ComponentUpgradePlan
+}
+
+// Certificate represents information for a certificate or a certificate authority when using the check-expiration command.
+type Certificate struct {
+	// Name of the certificate.
+	Name string
+
+	// ExpirationDate defines certificate expiration date in UTC following the RFC3339 format.
+	ExpirationDate metav1.Time
+
+	// ResidualTimeSeconds represents the duration in seconds relative to the residual time before expiration.
+	ResidualTimeSeconds int64
+
+	// ExternallyManaged defines if the certificate is externally managed.
+	ExternallyManaged bool
+
+	// CAName represents the name of the CA that signed the certificate.
+	// This field is empty for self-signed, root CA certificates.
+	CAName string
+
+	// Missing represents if the certificate is missing.
+	Missing bool
+}
+
+// +k8s:deepcopy-gen:interfaces=k8s.io/apimachinery/pkg/runtime.Object
+
+// CertificateExpirationInfo represents information for the output produced by 'kubeadm certs check-expiration'.
+type CertificateExpirationInfo struct {
+	metav1.TypeMeta
+
+	// Certificates holds a list of certificates to show expiration information for.
+	Certificates []Certificate
+
+	// CertificateAuthorities holds a list of certificate authorities to show expiration information for.
+	CertificateAuthorities []Certificate
 }
